package ru.landgrafhomyak.serdha.api.v0.ddl

import kotlin.jvm.JvmName
import ru.landgrafhomyak.serdha.api.v0.Expression

public interface TableCreator<TableUserExtension : Any> {
	public val expressionBuilder: Expression.Builder<TableUserExtension>

	public val types: ColumnType.Builder

	/**
	 * Offers column of type [D][type] with `NOT NULL` constraint named [name] and runtime type [RuntimeType].
	 *
	 * @return Descriptor of offered column for future operations.
	 */
	public fun <RuntimeType : Any, DatabaseType : ColumnType<RuntimeType>> column(name: String, type: DatabaseType): Column<RuntimeType, DatabaseType, TableUserExtension>

	public fun index(name: String, vararg columns: Column<*, *, TableUserExtension>): Index<TableUserExtension>

	public fun unique(name: String, distinctNulls: Boolean, vararg columns: Column<*, *, TableUserExtension>): UniqueConstraint<TableUserExtension>
	public fun check(name: String, constraint: Expression<Boolean, ColumnType.BOOLEAN, TableUserExtension>): CheckConstraint<TableUserExtension>

	public fun rowIdColumn(): Column<RowId<TableUserExtension>, ColumnType<RowId<TableUserExtension>>, TableUserExtension>

	public fun rowIdUniqueConstraint(): UniqueConstraint<TableUserExtension>

	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>> foreignKey(
		fromColumn: Column<*, ColumnType1, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn: Column<*, ColumnType1, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper>

	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>, ColumnType2 : ColumnType<*>> foreignKey(
		fromColumn1: Column<*, ColumnType1, TableUserExtension>,
		fromColumn2: Column<*, ColumnType2, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn1: Column<*, ColumnType1, TargetTableUserWrapper>,
		toColumn2: Column<*, ColumnType2, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper>

	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>, ColumnType2 : ColumnType<*>, ColumnType3 : ColumnType<*>> foreignKey(
		fromColumn1: Column<*, ColumnType1, TableUserExtension>,
		fromColumn2: Column<*, ColumnType2, TableUserExtension>,
		fromColumn3: Column<*, ColumnType3, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn1: Column<*, ColumnType1, TargetTableUserWrapper>,
		toColumn2: Column<*, ColumnType2, TargetTableUserWrapper>,
		toColumn3: Column<*, ColumnType3, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper>
}